(*********************************************************)
(* Fill class for background of TSLXPolarChart component *)
(* distributed for free, non-commercial purpose          *)
(*  2007 by Sl-Alex http://www.sl-alex.nm.ru/           *)
(*********************************************************)
unit PolarFill;

interface

uses Classes, Graphics, Types, Windows, ExtCtrls, JPEG;


type

//   - ,
TSLXFillStyle = (gsSolid,gsGradient,gsPicture);

//   - , 
TSLXDirection = (gdHorizontal,gdVertical);

//   
TSLXFill=class(TPersistent)
  private
    FOwner: TComponent;
    FStyle: TSLXFillStyle;
    FColor: TColor;
    FGradientStart: TColor;
    FGradientEnd: TColor;
    FGradientDirection: TSLXDirection;
    FGradientSteps: Cardinal;
    FPicture: TPicture;
    procedure FSetColor(NewColor: TColor);
    procedure FSetStyle(NewStyle: TSLXFillStyle);
    procedure FSetGradientStart(NewColor: TColor);
    procedure FSetGradientEnd(NewColor: TColor);
    procedure FSetDirection(NewDirection: TSLXDirection);
    procedure FSetGradientSteps(NewGradientSteps: Cardinal);
    procedure FSetPicture(NewPicture: TPicture);
    procedure FGradientFill(NewCanvas: TCanvas; NewRect: TRect; NewBeginColor, NewEndColor: TColor; NewStepsCount: Cardinal; NewDirection: TSLXDirection);
  public
    constructor Create(AOwner: TComponent);
    destructor Destroy; override;
  published
    procedure FillCanvas;
    property Color: TColor read FColor write FSetColor; //   
    property Style: TSLXFillStyle read FStyle write FSetStyle;//   (/)
    property GradientDirection: TSLXDirection read FGradientDirection write FSetDirection; //  
    property GradientStart: TColor read FGradientStart write FSetGradientStart; //   
    property GradientEnd: TColor read FGradientEnd write FSetGradientEnd;//   
    property GradientSteps: Cardinal read FGradientSteps write FSetGradientSteps;
    property Picture: TPicture read FPicture write FSetPicture;
end;

implementation

uses PolarChart;

constructor TSLXFill.Create(AOwner: TComponent);
begin
  inherited Create;
  FOwner:=AOwner;
  FStyle:=gsGradient;
  FColor:=clSilver;
  FGradientStart:=clGray;
  FGradientEnd:=clCream;
  FGradientDirection:=gdVertical;
  FGradientSteps:=255;
  FPicture:=TPicture.Create;
end;

destructor TSLXFill.Destroy;
begin
  FPicture.Destroy;
  inherited Destroy;
end;

procedure TSLXFill.FSetPicture(NewPicture: TPicture);
begin
  FPicture.Assign(NewPicture);
  (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXFill.FillCanvas;
begin
  //  
  if FStyle=gsGradient then begin
    FGradientFill((FOwner as TSLXPolarChart).BitMap.Canvas,Rect(0,0,(FOwner as TSLXPolarChart).Width,(FOwner as TSLXPolarChart).Height),FGradientStart,FGradientEnd,FGradientSteps,FGradientDirection);
  end;
  //  
  if FStyle=gsSolid then begin
    (FOwner as TSLXPolarChart).BitMap.Canvas.Brush.Color:=FColor;
    (FOwner as TSLXPolarChart).BitMap.Canvas.FillRect(Rect(0,0,(FOwner as TSLXPolarChart).Width,(FOwner as TSLXPolarChart).Height));
  end;
  //  
  if FStyle=gsPicture then begin 
    (FOwner as TSLXPolarChart).BitMap.Canvas.StretchDraw(Rect(0,0,(FOwner as TSLXPolarChart).Width,(FOwner as TSLXPolarChart).Height),FPicture.Graphic);
  end;
end;

//   
procedure TSLXFill.FGradientFill(NewCanvas: TCanvas; NewRect: TRect; NewBeginColor, NewEndColor: TColor; NewStepsCount: Cardinal; NewDirection: TSLXDirection);
var
  i, j, h, w: Integer;
  R, G, B: Longword;
  beginRGBvalue, RGBdifference: array[0..2] of LongInt;
begin
  beginRGBvalue[0] := GetRvalue(colortoRGB(NewBeginColor));
  beginRGBvalue[1] := GetGvalue(colortoRGB(NewBeginColor));
  beginRGBvalue[2] := GetBvalue(colortoRGB(NewBeginColor));

  RGBdifference[0] := GetRvalue(colortoRGB(NewEndColor)) - beginRGBvalue[0];
  RGBdifference[1] := GetGvalue(colortoRGB(NewEndColor)) - beginRGBvalue[1];
  RGBdifference[2] := GetBvalue(colortoRGB(NewEndColor)) - beginRGBvalue[2];

  NewCanvas.pen.Style := pssolid;
  NewCanvas.pen.mode := pmcopy;
  j := 0;

  h := NewRect.Bottom - NewRect.Top;
  w := NewRect.Right - NewRect.Left;

  R:=0;
  G:=0;
  B:=0;
  for i := NewStepsCount downto 0 do begin
    if NewDirection=gdHorizontal then begin
      NewRect.Left  := muldiv(i - 1, w, NewStepsCount);
      NewRect.Right := muldiv(i, w, NewStepsCount);
    end;
    if NewDirection=gdVertical then begin
      NewRect.Top  := muldiv(i - 1, h, NewStepsCount);
      NewRect.Bottom := muldiv(i, h, NewStepsCount);
    end;
    if NewStepsCount>0 then begin
      R := beginRGBvalue[0] + muldiv(j, RGBDifference[0], NewStepsCount-1);
      G := beginRGBvalue[1] + muldiv(j, RGBDifference[1], NewStepsCount-1);
      B := beginRGBvalue[2] + muldiv(j, RGBDifference[2], NewStepsCount-1);
    end;
    NewCanvas.Brush.Color := RGB(R, G, B);
    if NewDirection=gdHorizontal then begin
      patBlt(NewCanvas.Handle, NewRect.Left, NewRect.Top, NewRect.Right - NewRect.Left, h, patcopy);
    end;
    if NewDirection=gdVertical then begin
      patBlt(NewCanvas.Handle, NewRect.Left, NewRect.Top, w, NewRect.Bottom - NewRect.Top, patcopy);
    end;
    Inc(j);
  end;
end;

procedure TSLXFill.FSetColor(NewColor: TColor);
//    ,   gsSolid,   
begin
  FColor:=NewColor;
  if FStyle=gsSolid then (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXFill.FSetStyle(NewStyle: TSLXFillStyle);
//   
begin
  FStyle:=NewStyle;
  (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXFill.FSetGradientStart(NewColor: TColor);
//     ,   gsGradient,   
begin
  FGradientStart:=NewColor;
  if FStyle=gsGradient then (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXFill.FSetGradientEnd(NewColor: TColor);
//     ,   gsGradient,   
begin
  FGradientEnd:=NewColor;
  if FStyle=gsGradient then (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXFill.FSetDirection(NewDirection: TSLXDirection);
//    ,   gsGradient,   
begin
  FGradientDirection:=NewDirection;
  if FStyle=gsGradient then (FOwner as TSLXPolarChart).RepaintMem(True);
end;

procedure TSLXFill.FSetGradientSteps(NewGradientSteps: Cardinal);
//     ,   gsGradient,   
begin
  FGradientSteps:=NewGradientSteps;
  if FStyle=gsGradient then (FOwner as TSLXPolarChart).RepaintMem(True);
end;

end.
