(*********************************************************)
(* Series class for TSLXPolarChart component             *)
(* distributed for free, non-commercial purpose          *)
(*  2007 by Sl-Alex http://www.sl-alex.nm.ru/           *)
(*********************************************************)
unit PolarSeries;

interface

uses Classes, PolarSeriesPoint, ExtCtrls ,Graphics;

type

//  
TSLXPolarSeries = class(TPen) // 
  private
    { Private declarations }
    FPointList: TList; //  
    FMinimum: Double; //  
    FMaximum: Double; //  
    FCloseSeries: Boolean;
    FOwner: TComponent;
    function  FGetSeriesPointCount: Integer; //   
    procedure FPenChange(Sender: TObject); //   
    function  FGetPoint(const AIndex: Integer): TSLXPolarSeriesPoint; //    
  public
    constructor Create(AOwner: TComponent);// override; //  
    destructor Destroy; override; //  
    property Point[const AIndex: Integer]: TSLXPolarSeriesPoint read FGetPoint;// read GetSeries;
  published
    procedure Clear;                                         //  
    function AddPoint(angle, value: Double): Integer;                //  
    property Count: Integer read FGetSeriesPointCount;       //    
    property Minimum: Double read FMinimum; //    
    property Maximum: Double read FMaximum; // 
    property CloseSeries: Boolean read FCloseSeries write FCloseSeries; //   
end;

implementation

uses PolarChart;

constructor TSLXPolarSeries.Create(AOwner: TComponent);
begin
  inherited Create;
  FOwner:=AOwner;
  FPointList:=TList.Create;
  Color:=clRed;
  FCloseSeries:=False;
  OnChange:=FPenChange;
end;

destructor TSLXPolarSeries.Destroy;
begin
  //    
  Clear;
  //   
  FPointList.Destroy;
  //    :)
  inherited Destroy;
end;

procedure TSLXPolarSeries.FPenChange(Sender: TObject);
begin
  //   
  if FOwner<>nil then (FOwner as TSLXPolarChart).RepaintMem(True);
end;

function TSLXPolarSeries.AddPoint(angle, value: Double): Integer;
begin
  if FPointList.Count=0 then begin //  ,  -
    FMinimum:=value;
    FMaximum:=value;
  end else begin //     
    if value<FMinimum then begin
      FMinimum:=value;
    end;
    if value>FMaximum then begin
      FMaximum:=value;
    end;
  end;
  FPointList.Add(TSLXPolarSeriesPoint.Create(angle,value));
  AddPoint:=FPointList.Count;
  //  ,  
  if FOwner<>nil then (FOwner as TSLXPolarChart).RepaintMem(True);
end;

function TSLXPolarSeries.FGetSeriesPointCount: Integer;
begin
  FGetSeriesPointCount:=FPointList.Count;
end;

function TSLXPolarSeries.FGetPoint(const AIndex: Integer): TSLXPolarSeriesPoint;
begin
  if AIndex< FPointList.Count then
    FGetPoint:=FPointList.Items[AIndex]
  else
    FGetPoint:=nil;
end;

procedure TSLXPolarSeries.Clear;
var i: Integer;
  curPoint: TSLXPolarSeriesPoint;
begin
  if FPointList.Count>0 then begin
    //    
    for i:=0 to FPointList.Count-1 do begin
      curPoint:=FPointList.Items[i];
      curPoint.Destroy;
    end;
    //   
    FPointList.Clear;
  end;
end;

end.
